﻿package de.davidochmann.utilities.dogesture.logic 
{
	/**
	 * @author dochmann
	 */
	 
	import flash.events.TimerEvent;	
	import flash.utils.Timer;	 
	import flash.display.DisplayObject;	
	import flash.events.MouseEvent;	
	import flash.events.EventDispatcher;		
	import de.davidochmann.utilities.dogesture.stack.Stack;
	import de.davidochmann.utilities.dogesture.interfaces.IMove;	
	import de.davidochmann.utilities.dogesture.events.GestureEvent;	 
		
	public class Logic extends EventDispatcher
	{	
		private var mStack:Stack;
		private var mSurface:DisplayObject;
		private var mTimeout:uint;
		private var mType:String;
		
		private var mComplete:Boolean;
		private var mTimer:Timer;		

		private var mCurrent:IMove;

		private var mCompleteX:Boolean;
		private var mCompleteY:Boolean;

		private var mStartX:int;
		private var mStartY:int;
			
		private var mMoveX:int;
		private var mMoveY:int;
			
		private var mCurrentX:int;
		private var mCurrentY:int;
		

		public function Logic(){}
		
			
		public function start():void
		{
			initComplete();
			initSurfaceEvents();
			initTimer();
		}

		public function stop():void
		{

			mSurface.removeEventListener(MouseEvent.MOUSE_DOWN, surfaceMouseDownHandler);
			mSurface.removeEventListener(MouseEvent.MOUSE_UP, surfaceMouseUpHandler);
			mSurface.removeEventListener(MouseEvent.MOUSE_MOVE, surfaceMouseMoveHandler);
		}

		
		public function getStack():Stack
		{
			return mStack;
		}

		public function getSurface():DisplayObject
		{
			return mSurface;
		}

		public function getTimeout():uint
		{
			return mTimeout;
		}

		public function getType():String
		{
			return mType;
		}

		
		public function setStack(pStack:Stack):void
		{
			mStack = pStack;
		}
		
		public function setSurface(pSurface:DisplayObject):void
		{
			mSurface = pSurface;		
		}

		public function setTimeout(pTimeout:uint):void
		{
			mTimeout = pTimeout;
		}

		public function setType(pType:String):void
		{
			mType = pType;
		}

		
		private function initComplete():void
		{
			mComplete = false;
		}

		private function initSurfaceEvents():void
		{	
			mSurface.addEventListener(MouseEvent.MOUSE_DOWN, surfaceMouseDownHandler);
			mSurface.addEventListener(MouseEvent.MOUSE_UP, surfaceMouseUpHandler);
		}
		
		private function initTimer():void
		{
			if(mTimeout == 0)
				return;
			
			mTimer = new Timer(mTimeout, 0);
			mTimer.addEventListener(TimerEvent.TIMER, timerTimerHandler);	
		}
		
		private function timerTimerHandler(e:TimerEvent):void
		{			
			mTimer.stop();
			mSurface.removeEventListener(MouseEvent.MOUSE_MOVE, surfaceMouseMoveHandler);
		}
		
		private function surfaceMouseDownHandler(e:MouseEvent):void
		{		
			if(mTimer != null)
				mTimer.start();
			
			mCurrent = mStack.active();
			mCurrent.setStartX(e.localX);
			mCurrent.setStartY(e.localY);

			mSurface.addEventListener(MouseEvent.MOUSE_MOVE, surfaceMouseMoveHandler);
		}

		private function surfaceMouseUpHandler(e:MouseEvent):void
		{	
			mSurface.removeEventListener(MouseEvent.MOUSE_MOVE, surfaceMouseMoveHandler);
			
			if(mComplete)
			{
				dispatchEvent(new GestureEvent(GestureEvent.GESTURE_COMPLETE, mComplete, mStack.index(), mStack.list()));
				mComplete = false;
			}

			mStack.reset();
		}

		private function surfaceMouseMoveHandler(e:MouseEvent):void
		{
			e.updateAfterEvent();
			
			
			mCurrent:IMove;

			mCompleteX = false;
			mCompleteY = false;

			mStartX	   = mStack.active().getStartX();
			mStartY	   = mStack.active().getStartY();
			
			mMoveX	   = mStack.active().getPositionX();
			mMoveY	   = mStack.active().getPositionY();
			
			mCurrentX  = e.localX;
			mCurrentY  = e.localY;

			
			mCompleteX = (mMoveX != 0) ? ((mMoveX > 0) ? (mCurrentX - mStartX > mMoveX) : (mCurrentX - mStartX < mMoveX)) : true;
			mCompleteY = (mMoveY != 0) ? ((mMoveY > 0) ? (mCurrentY - mStartY > mMoveY) : (mCurrentY - mStartY < mMoveY)) : true;
					
			mComplete  = (mCompleteX && mCompleteY);
			
			
			if(mComplete) 
				mStack.next();
				
				
			if(mComplete && !mStack.complete())
			{
				mComplete = false;
				
				mCurrent  = mStack.active();
				mCurrent.setStartX(e.localX);
				mCurrent.setStartY(e.localY);
				
				dispatchEvent(new GestureEvent(GestureEvent.MOVE_COMPLETE, mComplete, mStack.index(), mStack.list()));
			}
			else if(mComplete && mStack.complete())
			{
				if(mTimer != null)
					mTimer.stop();
				
				mSurface.removeEventListener(MouseEvent.MOUSE_MOVE, surfaceMouseMoveHandler);
				dispatchEvent(new GestureEvent(GestureEvent.MOVE_COMPLETE, mComplete, mStack.index(), mStack.list()));
			}			
		}
	}
}
