﻿/**
 * @author dochmann
 */
import mx.utils.Delegate;
import mx.events.EventDispatcher;		
import de.davidochmann.utilities.dogesture.move.Move; 
import de.davidochmann.utilities.dogesture.stack.Stack;
import de.davidochmann.utilities.dogesture.register.Register;
import de.davidochmann.utilities.dogesture.events.GestureEvent;	 
import de.davidochmann.utilities.dogesture.register.events.RegisterEvent;
	
class de.davidochmann.utilities.dogesture.logic.Logic
{		
	private var mStack:Stack;
	private var mSurface:MovieClip;
	private var mTimeout:Number;
	
	private var mComplete:Boolean;
	private var mRegister:Register;
	private var mTimer:Number;		

	private var mCurrent:Move;

	private var mCompleteX:Boolean;
	private var mCompleteY:Boolean;

	private var mStartX:Number;
	private var mStartY:Number;
		
	private var mMoveX:Number;
	private var mMoveY:Number;
		
	private var mCurrentX:Number;
	private var mCurrentY:Number;
	
	
	private var mSurfaceMouseMove:Function;
	private var mSurfaceMouseDown:Function;
	private var mSurfaceMouseUp:Function;
	

	public function Logic()
	{
		super();
		EventDispatcher.initialize(this);
	}
	
		
	public function start():Void
	{
		initTimer();
		initComplete();
		initRegister();
		initFunctionReference();
		initSurfaceEvents();
	}

	public function stop():Void
	{
		mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_MOVE, mSurfaceMouseMove);	
		mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_DOWN, mSurfaceMouseDown);
		mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_UP  , mSurfaceMouseUp);
	}

	
	public function getStack():Stack
	{
		return mStack;
	}
	
	public function getTimeout():Number
	{
		return mTimeout;
	}

	public function getSurface():MovieClip
	{
		return mSurface;
	}

	
	public function setStack(pStack:Stack):Void
	{
		mStack = pStack;
	}

	public function setTimeout(pTimeout:Number):Void
	{
		mTimeout = pTimeout;
	}

	public function setSurface(pSurface:MovieClip):Void
	{
		mSurface = pSurface;
	}

	
	private function initTimer():Void
	{
		clearInterval(mTimer);
	}
	
	private function initComplete():Void
	{
		mComplete = false;
	}

	private function initRegister():Void
	{
		mRegister = Register.getInstance();
	}

	private function initFunctionReference():Void
	{
		mSurfaceMouseMove = Delegate.create(this, surfaceMouseMoveHandler);
		mSurfaceMouseDown = Delegate.create(this, surfaceMouseDownHandler);
		mSurfaceMouseUp   = Delegate.create(this, surfaceMouseUpHandler);
	}

	private function initSurfaceEvents():Void
	{
		mRegister.addEventListener(RegisterEvent.MOUSE_GESTURE_DOWN, mSurfaceMouseDown);
		mRegister.addEventListener(RegisterEvent.MOUSE_GESTURE_UP  , mSurfaceMouseUp);
	}

	private function timerTimerHandler():Void
	{	
		clearInterval(mTimer);
		mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_MOVE, mSurfaceMouseMove);	
	}
	
	private function surfaceMouseDownHandler():Void
	{
		if(mTimeout != undefined && mTimeout != 0)
			mTimer = setInterval(this, "timerTimerHandler", mTimeout);
		
		mCurrent = Move(mStack.active());
		mCurrent.setStartX(mSurface._xmouse);
		mCurrent.setStartY(mSurface._ymouse);

		mRegister.addEventListener(RegisterEvent.MOUSE_GESTURE_MOVE, mSurfaceMouseMove);
	}

	private function surfaceMouseUpHandler():Void
	{
		mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_MOVE, mSurfaceMouseMove);
		
		if(mComplete)
		{
			dispatchEvent(new GestureEvent(GestureEvent.GESTURE_COMPLETE, this, mComplete, mStack.index(), mStack.list()));
			mComplete = false;
		}
		
		mStack.reset();
	}

	private function surfaceMouseMoveHandler():Void
	{
		mComplete  = false;
		
		mCompleteX = false;
		mCompleteY = false;

		mStartX	   = mStack.active().getStartX();
		mStartY	   = mStack.active().getStartY();
		
		mMoveX	   = mStack.active().getPositionX();
		mMoveY	   = mStack.active().getPositionY();

		mCurrentX  = mSurface._xmouse;
		mCurrentY  = mSurface._ymouse;

		mCompleteX = (mMoveX != 0) ? ((mMoveX > 0) ? (mCurrentX - mStartX > mMoveX) : (mCurrentX - mStartX < mMoveX)) : true;
		mCompleteY = (mMoveY != 0) ? ((mMoveY > 0) ? (mCurrentY - mStartY > mMoveY) : (mCurrentY - mStartY < mMoveY)) : true;
				
		mComplete  = (mCompleteX && mCompleteY);
		
		
		if(mComplete) mStack.next();
	
				
		if(mComplete && !mStack.complete())
		{
			mComplete = false;
			
			mCurrent  = mStack.active();
			mCurrent.setStartX(mSurface._xmouse);
			mCurrent.setStartY(mSurface._ymouse);
			
			dispatchEvent(new GestureEvent(GestureEvent.MOVE_COMPLETE, this, mComplete, mStack.index(), mStack.list()));
		}
		else if(mComplete && mStack.complete())
		{
			clearInterval(mTimer);
			mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_MOVE, mSurfaceMouseMove);
			
			dispatchEvent(new GestureEvent(GestureEvent.MOVE_COMPLETE, this, mComplete, mStack.index(), mStack.list()));
		}			
	}
	
	
	public function addEventListener(type:String, listener:Object):Void{};
	
	public function removeEventListener(type:String, listener:Object):Void{};
	
	private function dispatchEvent(event:Object):Void{};
}