﻿package de.davidochmann.utilities.dogesture 
{
	/**
	 * DoGesture
	 * @version v.1.01
	 * @author 	David Ochmann
	 * 
	 * @mail	mail@davidochmann.de 	
	 * @website www.davidochmann.de
	 * 
	 * 
	 * 
	 * v1.01
	 * - Bugfix by cerupcat: Added mComplete = false after GestureEvent.GESTURE_COMPLETE dispatch in the Logic class.
	 *   Prevents DoGesture to trigger the Event after completing a gesture and afterwords clicking the parsed surface. 
	 * 
	 */
	 

	import flash.display.DisplayObject;	
	import flash.events.EventDispatcher;	
	import de.davidochmann.utilities.dogesture.move.Move;	 
	import de.davidochmann.utilities.dogesture.logic.Logic;	
	import de.davidochmann.utilities.dogesture.stack.Stack;	
	import de.davidochmann.utilities.dogesture.interfaces.IGesture;	
	import de.davidochmann.utilities.dogesture.events.GestureEvent;	

	public class DoGesture extends EventDispatcher implements IGesture 
	{	
		private var mSurface:DisplayObject;
		private var mTimeout:uint;
		private var mMoveArray:Array;
		private var mType:String;
		
		private var mStack:Stack;
		private var mLogic:Logic;
		

		public function DoGesture(pSurface:DisplayObject,
								  pTimeout:uint    = 0,
								  pMoveArray:Array = null,
								  pStart:Boolean   = false)
		{
			setSurface(pSurface);
			setTimeout(pTimeout);
			setMoveArray(pMoveArray);
			
			initStack();
			initLogic();
			initEventHandler();
			initMoveArray();
						
			if(pStart)
				start();
		}
		
		
		
		public function start():void
		{
			mLogic.setSurface(mSurface);
			mLogic.setStack(mStack);
			mLogic.setTimeout(mTimeout);
			mLogic.setType(mType);
			
			mLogic.start();
		}
		
		public function stop():void
		{
			mLogic.stop();
		}
		
		public function add(pPositionX:int, pPositionY:int):void
		{			
			var move:Move = new Move(pPositionX, pPositionY);
			mStack.add(move);
		}

		
		public function getSurface():DisplayObject
		{
			return mSurface;
		}

		public function getTimeout():uint
		{
			return mTimeout;
		}

		public function getMoveArray():Array
		{
			return mMoveArray;
		}

		
		public function setSurface(pSurface:DisplayObject):void
		{
			mSurface = pSurface;
		}

		public function setTimeout(pTimeout:uint):void
		{
			mTimeout = pTimeout;
		}
		
		public function setMoveArray(pMoveArray:Array):void
		{
			mMoveArray = pMoveArray;
		}

		
		private function initStack():void
		{
			mStack = new Stack();
		}
		
		private function initLogic():void
		{
			mLogic = new Logic();	
		}
		
		private function initEventHandler():void
		{
			mLogic.addEventListener(GestureEvent.MOVE_COMPLETE, logicMoveCompleteHandler);
			mLogic.addEventListener(GestureEvent.GESTURE_COMPLETE, logicMoveCompleteHandler);
		}
		
		private function initMoveArray():void
		{
			if(mMoveArray == null)
				return;
				
			var obj:Object;
			var move:Move;
				
			for(var i:uint = 0; i < mMoveArray.length; ++i)
			{
				obj  = Object(mMoveArray[i]);
				move = new Move(obj.x, obj.y);
				
				mStack.add(move);
			}			
		}
		
		
		private function logicMoveCompleteHandler(e:GestureEvent):void
		{
			dispatchEvent(e);
		}
	}
}
